import pytest
from emitter import Emitter


# testing the special events:
# Emitter.ATTACH
# Emitter.DETACH
# Emitter.ERROR


# Emitter.ATTACH


def test_attach__1():
    """
    ATTACH listeners are triggered on their own registration.
    """
    emitter = Emitter()
    l = []

    def attach_listener(event, listener):
        l.append(1)

    emitter.on(Emitter.ATTACH, attach_listener)

    assert len(l) == 1


def test_attach__2():
    """
    ATTACH listeners get the event and the listener that have been registered.
    """
    emitter = Emitter()
    d = {}

    # ATTACH callback receives the event and the listener
    def attach_listener(event, listener):
        d["event"] = event
        d["listener"] = listener

    emitter.on(Emitter.ATTACH, attach_listener)
    emitter.on("event", callable)

    assert d["event"] == "event"
    assert d["listener"] == callable


def test_attach__3():
    """
    ATTACH event is triggered even on listeners updates.
    """
    emitter = Emitter()
    l = []

    emitter.on(Emitter.ATTACH, lambda event, listener: l.append(1))

    emitter.on("event", callable)
    emitter.on("event", callable)

    assert len(l) == 3


# Emitter.DETACH


def test_detach__1():
    """
    DETACH event is triggered when removing a listener.
    """
    emitter = Emitter()
    l = []

    emitter.on(Emitter.DETACH, lambda err, *args, **kwargs: l.append(1))
    emitter.on("event", callable)

    emitter.off("event", callable)

    assert len(l) == 1


def test_detach__2():
    """
    DETACH event is not triggered if listener does not exists.
    """
    emitter = Emitter()
    l = []

    emitter.on(Emitter.DETACH, lambda err, *args, **kwargs: l.append(1))

    emitter.off("event", callable)

    assert len(l) == 0


def test_detach__3():
    """
    DETACH event is triggered for each listener when removing a whole event.
    """
    emitter = Emitter()
    l = []

    emitter.on(Emitter.DETACH, lambda err, *args, **kwargs: l.append(1))

    emitter.on("event", callable)
    emitter.on("event", bool)

    emitter.off("event")

    assert len(l) == 2


def test_detach__4():
    """
    DETACH is triggered for each listener of each event.
    """
    emitter = Emitter()
    l = []

    emitter.on(Emitter.DETACH, lambda err, *args, **kwargs: l.append(1))

    emitter.on("event1", callable)
    emitter.on("event1", bool)
    emitter.on("event2", callable)
    emitter.on("event2", bool)

    emitter.off()

    assert len(l) == 5


def test_detach__5():
    """
    DETACH listener receives the event and the listener affected.
    """
    emitter = Emitter()
    d = {}

    def detach_listener(event, listener):
        d["event"] = event
        d["listener"] = listener

    emitter.on(Emitter.DETACH, detach_listener)
    emitter.on("event", callable)

    emitter.off("event", callable)

    assert d["event"] == "event"
    assert d["listener"] == callable


# Emitter.ERROR


def test_error__1():
    """
    ERROR event is emitted when listener raises exception.
    """
    emitter = Emitter()
    l = []

    def listener(*args, **kwargs):
        raise Exception()

    emitter.on("thing", listener)
    emitter.on(Emitter.ERROR, lambda err: l.append(1))

    emitter.emit("thing")
    assert len(l) == 1


def test_error__2():
    """
    ERROR event handler gets error, *args and **kwargs.
    """
    emitter = Emitter()
    d = {}

    def listener(*args, **kwargs):
        raise Exception()

    def handler(err, *args, **kwargs):
        d["err"] = err
        d["args"] = args
        d["kwargs"] = kwargs

    emitter.on("thing", listener)
    emitter.on(Emitter.ERROR, handler)

    emitter.emit("thing", 10, b=20)

    assert isinstance(d["err"], Exception)
    assert d["args"][0] == 10
    assert d["kwargs"]["b"] == 20


def test_error__3():
    """
    If ERROR event handler raises exception, it is re-raised, and Emitter
    does not emit the ERROR event.
    """
    emitter = Emitter()
    l = []

    def listener(*args, **kwargs):
        raise Exception()

    def handler(err, *args, **kwargs):
        l.append(1)
        raise StopIteration()

    emitter.on(Emitter.ERROR, handler)
    emitter.on("event", listener)

    with pytest.raises(StopIteration):
        emitter.emit("event")

    assert len(l) == 1


def test_error__4():
    """
    One time ERROR listener is removed even if it raises exception.
    """
    emitter = Emitter()
    l = []

    def listener(*args, **kwargs):
        raise Exception()

    def handler(err, *args, **kwargs):
        l.append(1)
        raise StopIteration()

    emitter.once(Emitter.ERROR, handler)
    emitter.on("event", listener)

    assert len(emitter.listeners(Emitter.ERROR)) == 1

    with pytest.raises(StopIteration):
        emitter.emit("event")

    assert len(emitter.listeners(Emitter.ERROR)) == 0





import pytest
from emitter import Emitter


# Emitter()


# emitter.on()


def test_on__1():
    """
    None event raises an exception.
    """
    emitter = Emitter()

    with pytest.raises(ValueError):
        emitter.on(None, callable)


def test_on__2():
    """
    True is a valid event.
    """
    emitter = Emitter()
    emitter.on(True, callable)

    assert True in emitter.events()


def test_on__3():
    """
    False is a valid event.
    """
    emitter = Emitter()
    emitter.on(False, callable)

    assert False in emitter.events()


def test_on__4():
    """
    A string is a valid event.
    """
    emitter = Emitter()
    emitter.on("event", callable)

    assert "event" in emitter.events()


def test_on__5():
    """
    A listener cannot be None.
    """
    emitter = Emitter()

    with pytest.raises(TypeError):
        emitter.on("event", None)


def test_on__6():
    """
    A listener must be callable.
    """
    emitter = Emitter()

    with pytest.raises(TypeError):
        emitter.on("event", "")


def test_on__7():
    """
    Multiple events can be registered.
    """
    emitter = Emitter()

    emitter.on("event1", callable)
    emitter.on("event2", callable)

    assert "event1" in emitter.events()
    assert "event2" in emitter.events()


def test_on__8():
    """
    Listeners are not shared between events.
    """
    emitter = Emitter()

    emitter.on("event1", str)
    emitter.on("event2", callable)

    assert str in emitter.listeners("event1")
    assert str not in emitter.listeners("event2")

    assert callable in emitter.listeners("event2")
    assert callable not in emitter.listeners("event1")


def test_on__9():
    """
    Listeners can be called infinitely.
    """
    emitter = Emitter()
    l = []
    emitter.on("event", lambda: l.append(1))

    emitter.emit("event")
    emitter.emit("event")
    emitter.emit("event")

    assert len(l) == 3


def test_on__10():
    """
    Returns True when event has been successfully registered.
    """
    emitter = Emitter()
    assert emitter.on("event", callable) is True


def test_on__11():
    """
    Returns True when event has been successfully registered.
    """
    emitter = Emitter()
    assert emitter.on("event", callable) is True


def test_on__12():
    """
    Allow updating an event from on() to once().
    """
    emitter = Emitter()
    emitter.once("event", callable)
    emitter.on("event", callable)

    emitter.emit("event")
    emitter.emit("event")
    emitter.emit("event")

    assert callable in emitter.listeners("event")


# emitter.once()


def test_once__1():
    """
    Listener can be called only once.
    """
    emitter = Emitter()
    l = []
    emitter.once("event", lambda: l.append(1))
    emitter.emit("event")
    emitter.emit("event")
    emitter.emit("event")

    assert len(l) == 1


def test_once__2():
    """
    Listener should be removed after call.
    """
    emitter = Emitter()
    emitter.once("event", callable)
    emitter.on("event", bool)
    emitter.emit("event")

    assert callable not in emitter.listeners("event")
    assert bool in emitter.listeners("event")


def test_once__3():
    """
    Event should be cleaned if no more listeners.
    """
    emitter = Emitter()
    emitter.once("event", callable)
    emitter.emit("event")

    assert emitter.events() == set()


def test_once__4():
    """
    Allow updating an event from once() to on().
    """
    emitter = Emitter()
    emitter.on("event", callable)
    emitter.once("event", callable)

    emitter.emit("event")

    assert callable not in emitter.listeners("event")


def test_once__5():
    """
    One time listeners should be removed even if an error happens.
    """
    emitter = Emitter()

    def listener(*args, **kwargs):
        raise Exception()

    emitter.once("event", listener)

    emitter.emit("event")

    assert listener not in emitter.listeners("event")


# emitter.emit()


def test_emit__1():
    """
    All the listeners of an event must be triggered.
    """
    emitter = Emitter()
    l = []
    emitter.on("event", lambda: l.append(1))
    emitter.on("event", lambda: l.append(1))
    emitter.on("event", lambda: l.append(1))
    emitter.emit("event")
    assert len(l) == 3


def test_emit__2():
    """
    Listeners are triggered in order of insertion.
    """
    emitter = Emitter()
    l = []
    emitter.on("event", lambda: l.append(1))
    emitter.on("event", lambda: l.append(2))
    emitter.on("event", lambda: l.append(3))
    emitter.emit("event")
    assert l == [1, 2, 3]


def test_emit__3():
    """
    Only the listeners of the specified event should be triggered.
    """
    emitter = Emitter()
    l = []
    emitter.on("event1", lambda: l.append(1))
    emitter.on("event2", lambda: l.append(2))
    emitter.emit("event1")
    assert l == [1]


def test_emit__4():
    """
    Returns False when emitting a non-existent event.
    """
    emitter = Emitter()
    result = emitter.emit("event")
    assert result is False


def test_emit__5():
    """
    Returns True when emitting an event.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    result = emitter.emit("event")
    assert result is True


def test_emit__6():
    """
    Should pass *args and **kwargs to the listeners.
    """
    emitter = Emitter()
    params = []

    def listener(param1, param2, unused=None, param3=None):
        params.append(param1)
        params.append(param2)
        params.append(unused)
        params.append(param3)

    emitter.on("event", listener)
    emitter.emit("event", 10, 20, param3="hello")
    assert params == [10, 20, None, "hello"]


def test_emit__7():
    """
    False event can be emitted.
    """
    emitter = Emitter()
    l = []
    emitter.on(False, lambda: l.append(1))
    emitter.emit(False)
    assert 1 in l


def test_emit__8():
    """
    True event can be emitted.
    """
    emitter = Emitter()
    l = []
    emitter.on(True, lambda: l.append(1))
    emitter.emit(True)
    assert 1 in l


def test_emit__9():
    """
    Emitting None event returns False.
    """
    emitter = Emitter()

    assert emitter.emit(None) is False


# emitter.events()


def test_events__1():
    """
    Returns a empty set if no events registered.
    """
    emitter = Emitter()
    assert emitter.events() == set()


def test_events__2():
    """
    Returns a set containing all the registered events.
    """
    emitter = Emitter()
    emitter.on("event1", callable)
    emitter.on("event2", callable)
    emitter.on("event3", callable)
    events = emitter.events()
    assert events == {"event1", "event2", "event3"}


def test_events__3():
    """
    False event can be retrieved.
    """
    emitter = Emitter()
    emitter.on(False, callable)
    assert False in emitter.events()


def test_events__4():
    """
    True event can be retrieved.
    """
    emitter = Emitter()
    emitter.on(True, callable)
    assert True in emitter.events()


def test_events__5():
    """
    Should not return the original object, but a copy.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert emitter.events() is not emitter.events()


# emitter.listeners()


def test_listeners__1():
    """
    Returns an empty list when asking for an unknown event.
    """
    emitter = Emitter()

    assert isinstance(emitter.listeners(""), list)
    assert emitter.listeners("") == []


def test_listeners__2():
    """
    Returns a list containing all the listeners of the given event.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    emitter.on("event", list)

    listeners = emitter.listeners("event")

    assert isinstance(listeners, list)
    assert listeners == [callable, list]


def test_listeners__3():
    """
    The insertion order of the listeners should be conserved.
    """
    emitter = Emitter()
    emitter.on("raccoon", bool)
    emitter.on("raccoon", callable)
    emitter.on("raccoon", dict)

    listeners = emitter.listeners("raccoon")

    assert listeners == [bool, callable, dict]


def test_listeners__4():
    """
    Get the listeners for the False event.
    """
    emitter = Emitter()
    emitter.on(False, callable)
    assert callable in emitter.listeners(False)


def test_listeners__5():
    """
    Get the listeners for the True event.
    """
    emitter = Emitter()
    emitter.on(True, callable)
    assert callable in emitter.listeners(True)


def test_listeners__6():
    """
    Should not return the original object, but a copy.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert emitter.listeners("event") is not emitter.listeners("event")


# emitter.off()


def test_off__1():
    """
    Called with no arguments, it removes all the events.
    """
    emitter = Emitter()
    emitter.on("raccoon", callable)
    emitter.on("fox", callable)
    emitter.off()
    assert emitter.events() == set()


def test_off__2():
    """
    When called with 1 argument, it removes only the listeners of the
    specified event.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    emitter.on("event", str)
    emitter.on("raccoon", callable)
    emitter.on("raccoon", str)
    emitter.off("event")

    assert emitter.listeners("event") == []
    assert callable in emitter.listeners("raccoon")
    assert str in emitter.listeners("raccoon")


def test_off__3():
    """
    Called with 2 arguments, it removes the specified listener of the specified
    event.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    emitter.on("event", str)
    emitter.off("event", callable)
    listeners = emitter.listeners("event")
    assert callable not in listeners
    assert str in listeners


def test_off__4():
    """
    False event can be removed.
    """
    emitter = Emitter()
    emitter.on(False, callable)
    assert False in emitter.events()

    emitter.off(False)
    assert False not in emitter.events()


def test_off__5():
    """
    True event can be removed.
    """
    emitter = Emitter()
    emitter.on(True, callable)
    assert True in emitter.events()

    emitter.off(True)
    assert True not in emitter.events()


def test_off__6():
    """
    A listener of the False event can be removed.
    """
    emitter = Emitter()
    emitter.on(False, callable)
    assert callable in emitter.listeners(False)

    emitter.off(False, callable)
    assert callable not in emitter.listeners(False)


def test_off__7():
    """
    A listener of the True event can be removed.
    """
    emitter = Emitter()
    emitter.on(True, callable)
    assert callable in emitter.listeners(True)

    emitter.off(True, callable)
    assert callable not in emitter.listeners(True)


def test_off__8():
    """
    Returns True if all events are deleted.
    """
    emitter = Emitter()
    emitter.on("event1", callable)
    emitter.on("event2", callable)
    assert emitter.off() is True


def test_off__9():
    """
    Returns False if trying to remove a non-existent event.
    """
    emitter = Emitter()
    assert emitter.off("unknown") is False


def test_off__10():
    """
    Returns True if the specified event has been deleted.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert emitter.off("event") is True


def test_off__11():
    """
    Returns False if trying to detach a non-existent listener.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert emitter.off("event", bool) is False


def test_off__12():
    """
    Returns True if the specified listener has been detached.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert emitter.off("event", callable) is True


def test_off__13():
    """
    Delete the event if no more listeners.
    """
    emitter = Emitter()
    emitter.on("event", callable)
    assert "event" in emitter.events()

    emitter.off("event", callable)
    assert emitter.events() == set()




import collections


__version__ = "6.0.0"


class Emitter:
    # special events
    ERROR = object()
    ATTACH = object()
    DETACH = object()

    def __init__(self):
        self._events = {}

    def on(self, event, listener):
        # sanitize arguments types and values

        if event is None:
            raise ValueError("event cannot be None")

        if not callable(listener):
            raise TypeError("listener must be callable")

        # if the event doesn't exists yet, initialize it
        if event not in self._events:
            self._events[event] = collections.OrderedDict()

        # add listener to the event
        self._events[event].update({listener: {"once": False}})

        # emit attach event
        self.emit(Emitter.ATTACH, event, listener)

        return True

    def once(self, event, listener):
        # register the listener through the standard method
        result = self.on(event, listener)

        if not result:
            return result

        # update the "once" setting of the listener to True
        self._events[event].update({listener: {"once": True}})

        return True

    def off(self, event=None, listener=None):
        # if no event given, remove all events
        if event is None:
            # emit detach event for all the listener of all the events
            for event in self._events:
                for listener in self._events[event]:
                    self.emit(Emitter.DETACH, event, listener)

            # delete all events after detach event has been sent
            self._events = {}
            return True

        # if user tries to remove an non-existent event
        if self._events.get(event) is None:
            return False

        # delete all listeners for the given event
        if listener is None:
            # emit a detach event for all the listeners of the given event
            for listener in self._events[event]:
                self.emit(Emitter.DETACH, event, listener)

            # delete all listeners after detach event has been sent
            del self._events[event]
            return True

        # if user tries to remove a non-existent listener
        if self._events[event].get(listener) is None:
            return False

        # emit a detach event
        self.emit(Emitter.DETACH, event, listener)

        # delete the listener after detach event has been sent
        del self._events[event][listener]

        # if no more listeners in the given event, delete it
        if len(self._events[event]) == 0:
            del self._events[event]

        return True

    def events(self):
        # return a new set, containing events
        return set(self._events.keys())

    def listeners(self, event):
        # return a new list, containing listeners of the given event
        return list(self._events.get(event, []))

    def emit(self, event, *args, **kwargs):
        # if user tries to emit the None event
        if event is None:
            return False

        # if user tries to emits a non-existent event
        if self._events.get(event) is None:
            return False

        # trigger each listener attached to the event
        # we iterate on a copy to be allowed to mutate the OrderedDict during
        # iteration
        for listener in list(self._events[event]):
            # trigger the current listener
            try:
                listener(*args, **kwargs)
            except Exception as err:
                # if exception has been raised by error handler, re-raise it
                if event is Emitter.ERROR:
                    raise err

                # emit the error event
                self.emit(Emitter.ERROR, err, *args, **kwargs)
            finally:
                #  if emitter was "once", remove it
                if self._events[event][listener]["once"]:
                    self.off(event, listener)

        return True



from distutils.core import setup
setup(
  name = 'emitter.py',
  packages = ['emitter'],
  version = '6.0.0',
  description = 'A minimalist event emitter for Python 3.',
  author = 'titouandk',
  author_email = 'dk.titouan@gmail.com',
  url = 'https://github.com/titouandk/emitter.py',
  download_url = 'https://github.com/titouandk/emitter.py/tarball/6.0.0',
  keywords = ['event emitter', 'event', 'emitter', 'python 3', 'python3'],
  classifiers = ['Programming Language :: Python :: 3'],
)
